// PasswordDialog.java
// $Id: PasswordDialog.java,v 1.5 1999/10/12 08:44:31 bmahe Exp $
// (c) COPYRIGHT MIT, INRIA and Keio, 1999.
// Please first read the full copyright statement in file COPYRIGHT.html
 
package org.w3c.jwput.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JPasswordField;
import javax.swing.JPanel;
import javax.swing.JButton;
import javax.swing.BorderFactory;
import javax.swing.SwingUtilities;

/**
 * Winie's Password dialog
 * @version $Revision: 1.5 $
 * @author  Benot Mah (bmahe@w3.org)
 */
public class PasswordDialog extends JPanel implements JWDialog {

    protected String title    = null;
    protected String user     = null;
    protected String password = null;

    protected int returnValue = ERROR_OPTION;

    protected JDialog        dialog  = null;
    protected JTextField     userT   = null;
    protected JPasswordField passwdT = null;

    /**
     * Constructor
     * @param title the dialog title
     */
    public PasswordDialog(String title) {
	this.title    = title;
	this.user     = null;
	this.password = null;
	build();
    }

    protected void build() {
	ActionListener al = new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
		if( ae.getActionCommand().equals("Ok") || 
		    ae.getSource().equals(passwdT)) 
		{
		    user = userT.getText();
		    if (! user.equals("")) {
			returnValue = APPROVE_OPTION;
			password = new String(passwdT.getPassword());
			done();
		    } else {
			userT.requestFocus();
		    }
		} else if ( ae.getActionCommand().equals("Cancel")) {
		    returnValue = CANCEL_OPTION;
		    done();
		} else if(ae.getSource().equals(userT)) {
		    passwdT.requestFocus();
		}
	    }
	};

	GridBagLayout gbl = new GridBagLayout();
	GridBagConstraints gbc = new GridBagConstraints();
	GridBagLayout mgbl = new GridBagLayout();
	GridBagConstraints mgbc = new GridBagConstraints();
	JLabel l; JButton b;
	JPanel p = new JPanel(gbl);

	userT   = new JTextField(10);
	userT.setBorder(BorderFactory.createLoweredBevelBorder());

	passwdT = new JPasswordField(10);
	passwdT.addActionListener(al);
	passwdT.setBorder(BorderFactory.createLoweredBevelBorder());

	gbc.fill = GridBagConstraints.HORIZONTAL;
	gbc.weightx = 0;
	gbc.weighty = 0;
	mgbc.fill = GridBagConstraints.NONE;
	mgbc.weightx = 0;
	mgbc.weighty = 0;
	mgbc.insets = new Insets(16, 10, 16, 5);
	setLayout(mgbl);

	l = new JLabel("User name: ", JLabel.RIGHT);
	gbc.gridwidth = 1;
	gbl.setConstraints(l, gbc);
	p.add(l);
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(userT, gbc);
	p.add(userT);

	l = new JLabel("Password: ", JLabel.RIGHT);
	gbc.gridwidth = 1;
	gbl.setConstraints(l, gbc);
	p.add(l);
	gbc.gridwidth = GridBagConstraints.REMAINDER;
	gbl.setConstraints(passwdT, gbc);
	p.add(passwdT);
	mgbc.gridwidth = GridBagConstraints.REMAINDER;
	mgbl.setConstraints(p, mgbc);
	add(p);
	
	// and now the usual button bar
	p = new JPanel(new GridLayout(1, 2, 20, 20));
	b = new JButton("Ok");
	b.addActionListener(al);
	p.add(b);
	b = new JButton("Cancel");
	b.addActionListener(al);
	p.add(b);
	mgbl.setConstraints(p, mgbc);
	add(p);
	setBorder(BorderFactory.createTitledBorder("Authentication"));
    }

    protected void done() {
	if (dialog != null)
	    dialog.setVisible(false);
    }

    /**
     * Show this modal dialog
     * @param comp the parent component
     * @return CANCEL_OPTION, APPROVE_OPTION, ERROR_OPTION
     */
    public int showDialog(Component parent) {
	Frame frame = (parent instanceof Frame)
	    ? (Frame) parent
	    : (Frame)SwingUtilities.getAncestorOfClass(Frame.class, parent);

	dialog = new JDialog(frame, title, true);
	Container contentPane = dialog.getContentPane();
        contentPane.setLayout(new BorderLayout());
        contentPane.add(this, BorderLayout.CENTER);

	dialog.pack();
        dialog.setLocationRelativeTo(parent);
	//dialog.setSize(350,200);
        dialog.show();
	return returnValue;
    }

    /**
     * Get the password
     * @return a String instance
     */
    public String getPassword() {
	return password;
    }

    /**
     * Get the username
     * @return a String instance
     */
    public String getUser() {
	return user;
    }

}
